@@ -40,7 +40,9 @@ int ifd_init(void)
 	ifd_smph_register();
 	ifd_starkey_register();
 	ifd_towitoko_register();
+	ifd_rutoken_register();
 	/* ifd_wbeiuu_register();	driver not working yet */
+	ifd_cyberjack_register();	
 	/* ccid last */
 	ifd_ccid_register();
 
@@ -59,7 +61,7 @@ int ifd_init(void)
 	if (ifd_conf_get_integer("debug", &ival) >= 0 && ival > ct_config.debug)
 		ct_config.debug = ival;
 
-	if (ifd_conf_get_string("ifdhandler", &sval) >= 0)
+	if (ifd_conf_get_string("ifdhandler.program", &sval) >= 0)
 		ct_config.ifdhandler = sval;
 
 	/* Register all driver information (use hotplug ids) */
