@@ -17,15 +17,17 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/file.h>
+#include <dirent.h>
 #include <string.h>
 #include <stdio.h>
+#include <stdarg.h>
 #include <signal.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <fcntl.h>
 #include <errno.h>
 #include <limits.h>
-#ifdef HAVE_LIBUSB
+#ifdef ENABLE_LIBUSB
 #include <usb.h>
 #endif
 #include <openct/driver.h>
@@ -379,12 +381,38 @@ int ifd_sysdep_usb_open(const char *device)
         return open(device, O_RDWR);
 }
 
+#ifndef ENABLE_LIBUSB
+static int read_number (const char *read_format, const char *format, ...) {
+	va_list args;
+	char full[PATH_MAX];
+	FILE *fp = NULL;
+	int n = -1;
+
+	va_start(args, format);
+	vsnprintf (full, sizeof(full), format, args);
+	va_end(args);
+
+	if ((fp = fopen (full, "r")) == NULL) {
+		goto out;
+	}
+
+	fscanf (fp, read_format, &n);
+
+out:
+	if (fp != NULL) {
+		fclose (fp);
+	}
+
+	return n;
+}
+#endif
+
 /*
  * Scan all usb devices to see if there is one we support
  */
 int ifd_scan_usb(void)
 {
-#ifdef HAVE_LIBUSB
+#ifdef ENABLE_LIBUSB
 	ifd_devid_t id;
 	struct usb_bus *bus;
 	struct usb_device *dev;
@@ -397,7 +425,7 @@ int ifd_scan_usb(void)
 	id.num = 2;
 	for (bus = usb_busses; bus; bus = bus->next) {
 		for (dev = bus->devices; dev; dev = dev->next) {
-			const char *driver;
+			const char *driver = NULL;
 			char typedev[PATH_MAX];
 			struct stat buf;
 
@@ -410,45 +438,109 @@ int ifd_scan_usb(void)
 
 			if (!(driver = ifd_driver_for_id(&id))) {
 				/* no driver found, check for interface class */
-				int ccid=0;
 				int conf;
- 				for (conf = 0; conf < 
-					dev->descriptor.bNumConfigurations;
+ 				for (conf = 0; conf < dev->descriptor.bNumConfigurations;
 					conf++) {
-				int interf;
-  				for (interf = 0; interf <
-					dev->config[conf].bNumInterfaces;
-					interf++) {
-				int alt;
-				for (alt = 0; alt < dev->config[conf].interface[interf].num_altsetting; alt++) {
-					if (dev->config[conf].interface[interf].altsetting[alt].bInterfaceClass == 0x0b) {
-					ccid=1;
+					int interf;
+					for (interf = 0; interf < dev->config[conf].bNumInterfaces;
+						interf++) {
+						int alt;
+						for (alt = 0; alt < dev->config[conf].interface[interf].num_altsetting; alt++) {
+							if (dev->config[conf].interface[interf].altsetting[alt].bInterfaceClass == 0x0b) {
+								driver = "ccid";
+							}
+						}
+					}	
 				}
-				}
-				}	
-				}
-					
-				/* not a ccid device */
-				if (!ccid)
-					continue;
 			}
 
-			snprintf(typedev, sizeof(typedev),
-				 "/dev/bus/usb/%s/%s",
-				 bus->dirname, dev->filename);
-			if (stat(typedev, &buf) == 0) {
+			if (driver != NULL) {
 				snprintf(typedev, sizeof(typedev),
-				 	"usb:/dev/bus/usb/%s/%s",
-				 	bus->dirname, dev->filename);
-				ifd_spawn_handler(driver, typedev, -1);
-			} else {
-				snprintf(typedev, sizeof(typedev),
-				 	"usb:/proc/bus/usb/%s/%s",
-				 	bus->dirname, dev->filename);
-				ifd_spawn_handler(driver, typedev, -1);
+					 "/dev/bus/usb/%s/%s",
+					 bus->dirname, dev->filename);
+				if (stat(typedev, &buf) == 0) {
+					snprintf(typedev, sizeof(typedev),
+						"usb:/dev/bus/usb/%s/%s",
+						bus->dirname, dev->filename);
+					ifd_spawn_handler(driver, typedev, -1);
+				} else {
+					snprintf(typedev, sizeof(typedev),
+						"usb:/proc/bus/usb/%s/%s",
+						bus->dirname, dev->filename);
+					ifd_spawn_handler(driver, typedev, -1);
+				}
 			}
 		}
 	}
+#else
+	const char *base = "/sys/bus/usb/devices";
+	DIR *dir = NULL;
+	struct dirent *ent;
+
+	if ((dir = opendir (base)) == NULL) {
+		goto out;
+	}
+
+	while ((ent = readdir (dir)) != NULL) {
+		if (ent->d_name[0] != '.') {
+			int idProduct = -1;
+			int idVendor = -1;
+			int busnum = -1;
+			int devnum = -1;
+
+			idProduct = read_number ("%x", "%s/%s/%s", base, ent->d_name, "idProduct");
+			idVendor = read_number ("%x", "%s/%s/%s", base, ent->d_name, "idVendor");
+			busnum = read_number ("%d", "%s/%s/%s", base, ent->d_name, "busnum");
+			devnum = read_number ("%d", "%s/%s/%s", base, ent->d_name, "devnum");
+
+			ifd_debug (6, "coldplug: %s usb: %04x:%04x bus: %03d:%03d\n", ent->d_name, idProduct, idVendor, busnum, devnum);
+
+			if (idProduct != -1 && idVendor != -1 && busnum != -1 && devnum != -1) {
+				const char *driver = NULL;
+				ifd_devid_t id;
+
+				id.type = IFD_DEVICE_TYPE_USB;
+				id.num = 2;
+				id.val[0] = idVendor;
+				id.val[1] = idProduct;
+
+				if ((driver = ifd_driver_for_id(&id)) == NULL) {
+					DIR *dir1 = NULL;
+					struct dirent *ent1;
+
+					if ((dir1 = opendir (base)) != NULL) {
+						while ((ent1 = readdir (dir1)) != NULL && driver == NULL) {
+							/* skip all none bus elements */
+							if (strncmp (ent->d_name, ent1->d_name, strlen (ent->d_name))) {
+								continue;
+							}
+
+							int bInterfaceClass = read_number ("%x", "%s/%s/%s/%s", base, ent->d_name, ent1->d_name, "bInterfaceClass");
+
+							if (bInterfaceClass == 0x0b) {
+								driver = "ccid";
+							}
+						}
+						closedir (dir1);
+					}
+				}
+
+				if (driver != NULL) {
+					char typedev[1024];
+
+					snprintf(typedev, sizeof(typedev),
+						"usb:/dev/bus/usb/%03d/%03d",
+						busnum, devnum);
+					ifd_spawn_handler(driver, typedev, -1);
+				}
+			}
+		}
+	}
+
+out:
+	if (dir != NULL) {
+		closedir (dir);
+	}
 #endif
 	return 0;
 }
