@@ -93,13 +93,19 @@ int main(int argc, char **argv)
 
 static int mgr_init(int argc, char **argv)
 {
+	char *ifdhandler_user = NULL;
+	char *sval;
 	int n;
 
 	if (argc != 1)
 		usage(1);
 
+	/* Get the ifdhandler user so we can set ownership */
+	if (ifd_conf_get_string("ifdhandler.user", &sval) >= 0)
+		ifdhandler_user = sval;
+
 	/* Zap the status file */
-	ct_status_clear(OPENCT_MAX_READERS);
+	ct_status_clear(OPENCT_MAX_READERS, ifdhandler_user);
 
 	/* Initialize IFD library */
 	ifd_init();
@@ -151,6 +157,8 @@ static int mgr_shutdown(int argc, char **argv)
 			killed++;
 	}
 
+ 	ct_status_destroy();
+
 	printf("%d process%s killed.\n", killed, (killed == 1) ? "" : "es");
 	return 0;
 }
@@ -182,7 +190,7 @@ static int mgr_attach(int argc, char **argv)
 	sprintf(typedev, "%s:%s", type, device);
 
 	if (strncmp(type, driver, strlen(type)) == 0) {
-		/* detect what kind of devide we have */
+		/* detect what kind of device we have */
 
 		if (ifd_device_id_parse(driver, &id) < 0) {
 			fprintf(stderr, "Cannot parse device ID %s\n", driver);
